/*********************************/
/*    ナダ電子 AS-289R テスト01  */
/*********************************/
//    2014-04-21    下間憲行
//    2015-10-06    PROGMEMの書式を変更

/*****  ライブラリ      ****/
#include  "wiring_private.h"       // sbi,cbi命令用
// #include  <MsTimer2.h>          // 0.5msに対応していないので使わない
// #include  <avr/io.h>
// #include  <avr/pgmspace.h>      // PROGMEM用
// #include  <avr/interrupt.h>     // 割り込み用

/*****  I/Oピン指定    *****/
#define  SW_START  6        // start/stop SW
#define  SW_CH1    12       // CH1 on/off
#define  SW_CH2    13       // CH2 on/off
#define  SW_OFST   A4       // offset VR処理無し
                            // いずれもHでオン
/*****  マクロ        *****/
#define  DIMSIZ(a)  (sizeof(a) / sizeof(*a))  // 配列の大きさ

/*****  ピン制御マクロ    *****/
//      タイミングチェックパルス出力
#define  TP2_H    (sbi(PORTD,PORTD2))    // 1ms割り込み
#define  TP2_L    (cbi(PORTD,PORTD2))
#define  TP3_H    (sbi(PORTD,PORTD3))    // A/D割り込み
#define  TP3_L    (cbi(PORTD,PORTD3))
#define  TP4_H    (sbi(PORTD,PORTD4))    // loop実行時間
#define  TP4_L    (cbi(PORTD,PORTD4))
#define  TP4_TGL  (sbi(PIND, PIND4))

/******************************/
/*      イメージプリント処理  */
/******************************/

/*****  プリントデータバッファ    *****/
#define  P_BYTE    48              // 1行バイト数 384ドットのバイト数
byte  p_bff[P_BYTE];               // 1行分のバッファ
                                   // これをプリンタに送出

/*****  イメージプリントコマンド  *****/
const byte pcmd_bitimg[] PROGMEM ={
    0x1c, 0x2a, 0x63,      // イメージ登録+プリント
    0x00, 0x01,            // 1line
};

/*****  1ラインバッファ出力      *****/
void prnpbff(void)
{
byte i;
    for(i = 0; i < sizeof(pcmd_bitimg); i++){  // コマンド出力
        Serial.write(pgm_read_byte(&pcmd_bitimg[i]));
    }
    for(i = 0; i < P_BYTE; i++){          // バッファ出力
        Serial.write(p_bff[i]);          // 1行
    }
}

/*****  ビットイメージデータ  *****/
const byte bit_on[] PROGMEM ={
    0b10000000,       // 0
    0b01000000,       // 1
    0b00100000,       // 2
    0b00010000,       // 3
    0b00001000,       // 4
    0b00000100,       // 5
    0b00000010,       // 6
    0b00000001,       // 7
};

/*****  ビットイメージデータ(左側）  *****/
const byte bit_on1[] PROGMEM ={
    0b11111111,       // 0
    0b01111111,       // 1
    0b00111111,       // 2
    0b00011111,       // 3
    0b00001111,       // 4
    0b00000111,       // 5
    0b00000011,       // 6
    0b00000001,       // 7
};

/*****  ビットイメージデータ(右側）  *****/
const byte bit_on2[] PROGMEM ={
    0b10000000,       // 0
    0b11000000,       // 1
    0b11100000,       // 2
    0b11110000,       // 3
    0b11111000,       // 4
    0b11111100,       // 5
    0b11111110,       // 6
    0b11111111,       // 7
};

/*****  ドットセット      *****/
//    左上 0,0　　383,0 右上
//    左下 0,1　　383,1 右下
void pset(int x)
{
      p_bff[x / 8] |=
          pgm_read_byte(&bit_on[x % 8]);    // 1ビットオン
}

/*****  プリントバッファクリア      *****/
void clrpbff(void)
{
    memset(p_bff, 0, sizeof(p_bff));
}

/**********************************/
/*    プリント、紙送り処理        */
/**********************************/

/*****  紙送りデータ    *****/
#define  CM_MS    125      // 5秒/1cmの基準パルス
                           // 5秒/80パルス = 0.0625秒
                           // 0.5ms割り込みで125回
volatile byte tm_feed;     // 紙送り基準ﾀｲﾏｰ
                           // 1msでアップカウント
volatile word feed_cnt;    // feedplsと比較して1ピッチ紙送り
volatile byte f_feed;      // 紙送りフラグ
                           // タイマー割り込み内でオン
byte feed_8;               // 紙送り1mmチェック
                           // 1mmで8パルス
word feed_mm;              // 紙送りmm単位の長さ
                           // 00～999mmを繰り返す
byte  f_swstart;           // SW_STARTの状態
byte  f_swston;            // START オンのエッジ
byte  dsw_rot;             // 4bit DSWのデータ 0～9
byte  f_dswrotx;           // DSWデータが変化したときにオン
volatile byte  tm_swstck;  // SW_STARTﾁｪｯｸ用ﾀｲﾏｰ
volatile byte  tm_dswck;   // DSW ﾁｪｯｸ用ﾀｲﾏｰ

/*****  ROT DSWの入力    *****/
//    0～9を入力
byte rddsw(void)
{
byte d;
    d = (~PINB) & 0x0F;    // PB0～PB3(MSB)
    if(d > 9)    d = 0;
    return d;
}

/*****  紙送り速度      *****/
//    5秒/cmの紙送りパルス(62.5ms)を単位として指定
const word feedpls[] PROGMEM ={
      1,        // 0  5秒/cm
      2,        // 1  10秒/cm
      6,        // 2  30秒/cm
      12,       // 3   1分/cm
      60,       // 4   5分/cm
      120,      // 5  10分/cm
      360,      // 6  30分/cm
      720,      // 7   1時/cm
      4320,     // 8   6時/cm
      8640,     // 9  12時/cm
};

/*****  紙送り速度表示データ      *****/
const char spd0[] PROGMEM = "5sec/cm";
const char spd1[] PROGMEM = "10sec/cm";
const char spd2[] PROGMEM = "30sec/cm";
const char spd3[] PROGMEM = "1min/cm";
const char spd4[] PROGMEM = "5min/cm";
const char spd5[] PROGMEM = "10min/cm";
const char spd6[] PROGMEM = "30min/cm";
const char spd7[] PROGMEM = "1hour/cm";
const char spd8[] PROGMEM = "6hour/cm";
const char spd9[] PROGMEM = "12hour/cm";

const char * const spdmsg[] PROGMEM = {
      spd0,  spd1,  spd2,  spd3,  spd4,
      spd5,  spd6,  spd7,  spd8,  spd9,
};

//    用紙スケール
//    0   0   0   1   1   1   2   2   3   3   3  
//    0   3   7   1   5   9   2   6   0   4   8  
//    0   8   6   2   2   0   8   6   4   2   0
//    |                   .                   | 0.5mm
//    |   .   .   .   .   .   .   .   .   .   | 1mm
//    | . . . . . . . . . . . . . . . . . . . | 10mm

#define  P_SCL    380              // スケール上限
#define  P_CEN    190              // 中央
#define  P_PIT1    38              // 1mmごと

/*****   スケールを描画        *****/
void pscale(void)
{
int i;
    pset(0);                        // 左端
    pset(P_SCL);                    // 右端
    if((feed_8 == 0) && ((feed_mm % 10) == 0)){   // 10mmピッチ
        for(i = 0; i < P_BYTE; i++){              // 1cmのライン
            p_bff[i] = 0b11001100;
        }
    }  
    else if(feed_8 == 0){          // 1mmピッチ
        for(i = P_PIT1; i < P_SCL; i+=P_PIT1){
            pset(i);
        }
    }
    else if((feed_8 % 4) == 0){     // 0.5mmピッチ
        pset(P_CEN);                // 中央の点
    }     
}

/******************************/
/*      Ａ／Ｄ処理            */
/******************************/

/****  A/D データ        *****/
//    AD0  ch1データ入力
//    AD1  ch2データ入力
//    AD2  ch1オフセット調整VR入力
//    AD3  ch2オフセット調整VR入力
volatile byte  ad_ch;           // A/D変換チャンネル
                                // 0,1 : min,max処理
                                // 1,2 : 平均処理
//  AD0,AD1 データ (ch1,ch2,データ)
volatile int   ad_data[2];     // A/D変換データ 0～1023
volatile int   ad_max[2];      // A/D max
volatile int   ad_min[2];      // A/D min
//  AD2,AD3 データ (オフセットデータ)
volatile int   ad_offset[2];    // offset データ ±512
                                // 平均値
volatile word  ad_add[2];       // 平均加算値
volatile word  ad_cnt;          // 平均回数
//  A/D min,max更新フラグ
volatile byte f_adnext;         // A/Dmin,maxを次データに
                                // メイン側でオンして割り込みでクリア
//  offset VR 平均値確定フラグ
volatile byte f_ofstok;         // 平均値確定, 割り込みでオン

/*****  A/D 変換チャンネル指定    *****/
const byte ad_mux[] PROGMEM ={
    0b11000000,       // ch0 入力1
    0b11000001,       // ch1 入力2
    0b11000010,       // ch2 offset VR1
    0b11000011,       // ch3 offset VR2
//    ||| ++++----  MUX
//    ||+---------  ADLAR
//    ++----------  REFS   内部基準電圧1.1V
};

/*****  A/D割り込み処理    *****/
//    タイマー割り込みでA/D変換開始
//    4msで4chの変換を終わる
//    0,1は入力データ　2,3はオフセットVR
//    min,max判定のためこの処理の中で
//    ch1,ch2データにオフセットを加算する
ISR(ADC_vect)
{
int d;
    TP3_H;                      // (!!!)test
    d = (word)ADCL;             // 
    d |= (ADCH & 0x03) << 8;    // 符号なしで 0～3FF
//  測定値
    if(ad_ch < 2){              // ch1,ch2 max,minチェック
      d += ad_offset[ad_ch] ;         // offset VR値を加算
      ad_data[ad_ch] = d;             // データを保存
      if(d > ad_max[ad_ch])    ad_max[ad_ch] = d;    // max
      if(d < ad_min[ad_ch])    ad_min[ad_ch] = d;    // min
    }
//  offset VR
    else{                        // AD2,AD3 オフセット調整VR入力
      ad_add[ad_ch - 2] += d;    // 平均処理のためA/D値を加算
    }
//  4chとも変換完了
    ad_ch++;                          // 次ch
    if(ad_ch >= sizeof(ad_mux)){      // 4ch終了
        ad_ch = 0;                    // ch0に戻す
//  AD0,1 : フラグがあれば次データに更新
        if(f_adnext){           // 次変換データの処理
          ad_max[0] = ad_min[0] = ad_data[0];    // ch1入力
          ad_max[1] = ad_min[1] = ad_data[1];    // ch2入力
          f_adnext = 0;
        }
//  AD2,3 : オフセットVRは平均回数をチェック
        ad_cnt ++;              // 平均回数
        if(ad_cnt >= 64){       // 64回目
          if(digitalRead(SW_OFST) == HIGH){    // offset VR 処理なし
            ad_offset[0] = ad_offset[1] = 0;   // offset = 0
          }
          else{
            ad_offset[0] =                     // ch1 offset
                (int)((ad_add[0] / 64)) - 512; // ±512
            ad_offset[1] =                     // ch2 offset
                (int)((ad_add[1] / 64)) - 512; // ±512
          }
          ad_add[0] = ad_add[1] = 0;  // 加算値をゼロに
          ad_cnt = 0;          // 最初から
          f_ofstok = 1;        // 平均値確定で1
        }
    }
    TP3_L;
}

/******************************/
/*      タイマー割り込み      */
/******************************/

/*****  タイマー値    *****/
volatile byte  tm_1ms;       // 1msﾀﾞｳﾝｶｳﾝﾄﾀｲﾏｰ max 255ms

/*****  タイマー2 ０．５ｍＳ割り込み    *****/
//void tm2int(void)
ISR(TIMER2_COMPA_vect)
{
static byte tc = 0;                   // 1msカウント 
    TP2_H;                            // (!!!)test
//  0.5ms 紙送りタイマー
    if(f_swstart == HIGH){          // start/stop SW
      tm_feed++;                    // 1msアップカウント
        if(tm_feed >= CM_MS){       // 125msが基準 
          tm_feed = 0;
          feed_cnt++;             // 送りカウンタ
          if(feed_cnt >= pgm_read_word(&feedpls[dsw_rot])){  // DSW設定値
            feed_cnt = 0;
            f_feed = 1;         // 紙送りフラグをオン
          }
        }
    }
//  0.5msを2回カウントして1msに
    tc++;                    // 0,1
    if(tc >= 2){             // 1ms
      tc = 0;  
//  1ms計時処理（ダウンカウント）
      if(tm_1ms)      tm_1ms--;
      if(tm_swstck)   tm_swstck--;
      if(tm_dswck)    tm_dswck--;
//  A/D変換
      ADMUX = pgm_read_byte(&ad_mux[ad_ch]);  // ch指定
      ADCSRA |= (1 << ADSC);      // A/D変換開始
    }
    TP2_L;
}

/****************************************/
/*    スイッチ入力チェック              */
/****************************************/

/*****  START/STOPスイッチチェック    *****/
void ckswstart(void)
{
static byte ck = 0;
byte d;
    d = digitalRead(SW_START);
    switch(ck){
      case 0:
        if(d != f_swstart){    // 変化あり
          tm_swstck = 10;      // 10ms
          ck++;
        }
        break;
      case 1:
        if(d == f_swstart){    // 元に戻った
          ck = 0;
        }
        else{                         // 変化継続
          if(tm_swstck == 0){         // 10ms経過
            f_swstart = d;            // 新状態
            if(d == HIGH){
              f_swston = 1;           // onのエッジ
            }
            ck = 0;
          }
        }
        break;
    }
}

/*****  ROT-DSW入力      *****/
//    dsw_rotにデータを残す
void ckrotdsw(void)
{
static byte ck = 0;
static byte d;
byte r;
    r = rddsw();                   // DSW 設定値
    switch(ck){
      case 0:
        if(dsw_rot != r){          // 変化あり
          d = r;                   // 変化した状態
          tm_dswck = 100;          // 100ms
          ck++;
        }
        break;
      case 1:
        if(d != r){                // また変わった
          ck = 0;                  // 最初から
        }
        else{                      // 同じ状態継続
          if(tm_dswck == 0){       // 100ms経過
            dsw_rot = d;           // 新状態
            f_dswrotx = 1;         // 変化ありを知らせる
            ck = 0;
          }
        }
        break;
    }
}

/******************************/
/*    A/D 値スケーリング      */
/******************************/

/*****  A/D値からX位置を計算    *****/
//    in  a:A/D 0～1023 + offset(±512)
//    ret X位置 0～380
int xpos(int a)
{
long x;
    x  = (long)a * (long)P_SCL;  // 用紙幅とスケーリング
    x /= 1024L;
    if(x > P_SCL)    x = P_SCL;
    if(x < 0)        x = 0;
    return (int)x;
}

/*****  グラフ描画      *****/
//      x1,x2 座標
void pgrph(int x1, int x2)
{
int i, a1,b1, a2,b2;
    if(x1 > x2){        // x2を大きく
        a1 = x1;
        x1 = x2;
        x2 = a1;
    }
    if(x1 == x2)    x2+=1;    // 同じ点なら左へ拡張
// アドレスとビット位置
    a1 = x1 / 8;
    b1 = x1 % 8;
    a2 = x2 / 8;
    b2 = x2 % 8;
//  同じアドレスか
    if(a1 == a2){        // 同じアドレス
        p_bff[a1] |= 
          pgm_read_byte(&bit_on1[b1]) &
          pgm_read_byte(&bit_on2[b2]);      // 左右ビットをANDしてOR
    }
    else{               // 違うアドレス
        p_bff[a1] |= pgm_read_byte(&bit_on1[b1]);  // 左
        p_bff[a2] |= pgm_read_byte(&bit_on2[b2]);  // 右ビット
        if((a2 - a1) > 1){                        // 中間がある
          for(i = (a1 + 1); i < a2; i++){         // 間はFFで埋める
            p_bff[i] |= 0xff;
          }
        }
    }
}

#if 0      // psetでの描画
int  a;
    if(x1 > x2){
        a  = x1;
        x1 = x2;
        x2 = a;
    }
    a = x1;
    for(a = x1; a <= x2; a++){
      pset(a);
    }
#endif

/*****  紙送り速度を出力    *****/
void prnspeed(void)
{
char s[16];
    strcpy_P(s,(PGM_P)pgm_read_word(&spdmsg[dsw_rot]));  // 設定値
    Serial.print(s);              // 送り速度
    Serial.write('\r');           // 印字と改行
}

/**********************************/
/*    キャラジェネ                */
/**********************************/

/***** 数字 0～9    *****/
//    横8ドット 高16ドット
const byte cg09[] PROGMEM ={
    /* 30 (0) */
    0b00111100,
    0b01111110,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b01111110,
    0b00111100,
    /* 31 (1) */
    0b00011000,
    0b00011000,
    0b00111000,
    0b00111000,
    0b01111000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00111100,
    0b00111100,
    /* 32 (2) */
    0b00111100,
    0b01111110,
    0b11000011,
    0b11000011,
    0b11000011,
    0b00000011,
    0b00000011,
    0b00000110,
    0b00001100,
    0b00011000,
    0b00110000,
    0b01100000,
    0b11000000,
    0b11000000,
    0b11111111,
    0b11111111,
    /* 33 (3) */
    0b00111100,
    0b01111110,
    0b11000011,
    0b11000011,
    0b00000011,
    0b00000011,
    0b00000110,
    0b00111100,
    0b00111100,
    0b00000111,
    0b00000011,
    0b00000011,
    0b11000011,
    0b11000011,
    0b01111110,
    0b00111100,
    /* 34 (4) */
    0b00000110,
    0b00001110,
    0b00011110,
    0b00110110,
    0b00110110,
    0b01100110,
    0b01100110,
    0b11000110,
    0b11000110,
    0b11111111,
    0b11111111,
    0b00000110,
    0b00000110,
    0b00000110,
    0b00000110,
    0b00000110,
    /* 35 (5) */
    0b11111111,
    0b11111111,
    0b11000000,
    0b11000000,
    0b11000000,
    0b11000000,
    0b11111110,
    0b01111111,
    0b00000011,
    0b00000011,
    0b00000011,
    0b00000011,
    0b11000011,
    0b11000011,
    0b01111110,
    0b00111100,
    /* 36 (6) */
    0b00111100,
    0b01111110,
    0b11000011,
    0b11000000,
    0b11000000,
    0b11000000,
    0b11000000,
    0b11111100,
    0b11111110,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b01111110,
    0b00111100,
    /* 37 (7) */
    0b11111111,
    0b11111111,
    0b00000011,
    0b00000011,
    0b00000011,
    0b00000110,
    0b00000110,
    0b00001100,
    0b00001100,
    0b00001100,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    0b00011000,
    /* 38 (8) */
    0b00111100,
    0b01111110,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b01111110,
    0b01111110,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b01111110,
    0b00111100,
    /* 39 (9) */
    0b00111100,
    0b01111110,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b11000011,
    0b01111111,
    0b00111111,
    0b00000011,
    0b00000011,
    0b00000011,
    0b00000011,
    0b11000011,
    0b11111110,
    0b00111100,
};

/*****  1cmピッチプリント用データ  *****/
byte p1cm_cg1[16+4];    // 1cmピッチプリント用バッファ MSB側
byte p1cm_cg2[16+4];    // LSB側     8x16 キャラジェネをコピー
                        // 上下2ドットを空けておくために16+4
byte p1cm_ptr;          // lineバッファへのコピー位置
byte p1cm_cnt;          // コピーデータ数
                        // 0以外なら紙送りのタイミングで
                        // グラフの右端にプリント

/*****  紙送り1cmピッチプリント開始  *****/
//  9mmごとに8×16ドットでプリント
// 上下2ドットを空けておく
void prn1cmset(void)
{
word  m;
    m = (feed_mm / 10 )  % 100;         // 00～99cm
    memcpy_P(p1cm_cg1+2, &cg09[(m / 10) * 16], 16);  // MSB側CG
    memcpy_P(p1cm_cg2+2, &cg09[(m % 10) * 16], 16);  // LSB側CG
    p1cm_cnt = 20;        // 空白2.縦16ドット,空白2ドット
    p1cm_ptr = 0;         // 先頭ライン
}

/*****  紙送り1cmピッチプリント実行  *****/
//      lineバッファの右端(+45,+46,+47)にコピー
void prn1cmexc(void)
{
    p_bff[P_BYTE-1] = p1cm_cg2[p1cm_ptr];      // LSB CG 左端
    p_bff[P_BYTE-2] = p1cm_cg1[p1cm_ptr] << 2; // MSB CG 1/2右
    p_bff[P_BYTE-3] = p1cm_cg1[p1cm_ptr] >> 6; // MSB CG 1/2左
    p1cm_ptr++;                      // 次行へ
    p1cm_cnt--;                      // カウンタ -1
}

/******************************/
/*    セットアップとループ    */
/******************************/

/*****  セットアップ    ****/
void setup()
{
    Serial.begin(9600);
    pinMode(2,OUTPUT);        // PD2
    pinMode(3,OUTPUT);        // PD3
    pinMode(4,OUTPUT);        // PD4
    pinMode(SW_START,INPUT_PULLUP);  // START/STOP
    pinMode(SW_CH1,INPUT_PULLUP);    // CH1 on/off
    pinMode(SW_CH2,INPUT_PULLUP);    // CH2 on/off
    pinMode(SW_OFST,INPUT_PULLUP);   // offset VR on/off
    pinMode(8,INPUT_PULLUP);  // ROT DSW 1 PIB0
    pinMode(9,INPUT_PULLUP);  // ROT DSW 2 PINB1
    pinMode(10,INPUT_PULLUP); // ROT DSW 4 PINB2
    pinMode(11,INPUT_PULLUP); // ROT DSW 8 PINB3
//  A/D割り込み
    ADCSRA |= (1 << ADEN);      // A/D許可
    ADCSRA |= (1 << ADIE);      // A/D割り込み有効
//  ﾀｲﾏｰ計時
//    MsTimer2::set(1, tm2int);   // 1msｻｲｸﾙ割り込み処理
//    MsTimer2::start();
    TCCR2A = 0b00000010;
    //               ++----  WGM21,20 CTCモード
    TCCR2B = 0b00000100;
    //             |+++----  プリスケーラ1/64
    //             +-------  WGM22
    OCR2A  = 125-1;          // 16MHz / 64 / 125 = 2kHz
    TIMSK2 = 0b00000010;
    //               +-----  OCIE2A
    sei();                      // 割り込み開始
//  最初の測定
    f_ofstok = 0;              // offset A/D 平均値確定待ち
    while(f_ofstok == 0){      // 割り込みで1になるのを待つ
        TP4_TGL;               // (!!!)test
    }
    f_adnext = 1;               // A/D min,max更新指令
    while(f_adnext);            // 4ms経過して更新完了までwait
    f_feed = 0;                 // 最初の紙送りを無視
//  紙送り速度をプリント
    dsw_rot = rddsw();
    f_swstart = digitalRead(SW_START);    // 紙送り状態
    if(f_swstart == HIGH){      // start状態
        prnspeed();             // 紙送り速度をプリント
    }
}

/*****  ルー プ      *****/
void loop()
{
int d_max[2];        // max
int d_min[2];        // min
                     // A/D 測定データ ch1,ch2 +offset VR
int i;
//  スイッチチェック
    ckswstart();               // STARTスイッチ状態
    if(f_swston){              // onした
        f_swston = 0;
        prnspeed();            // 紙送り速度をプリント
        feed_8  = 0;           // 0mmから
        feed_mm = 0;
        f_feed  = 0;           // 紙送りをキャンセル
    }
    ckrotdsw();                // 速度設定DSWの変化をチェック
    if(f_dswrotx){             // 変化があれば
        f_dswrotx = 0;
        if(f_swstart == HIGH){   // 印刷中なら
          prnspeed();            // 紙送り速度をプリント
        }
    }
//  紙送り
    if(f_feed){                // 紙送り指令
        TP4_H;                 // (!!!)test
        f_feed = 0;
        clrpbff();             // プリントバッファをクリア
        pscale();              // スケールを描画
//  スケールcmをプリント
        if((feed_8 == 1) && ((feed_mm % 10) == 0)){  // 10mmごと
            prn1cmset();                       // CGデータをセット
        }
        if(p1cm_cnt)           prn1cmexc();    // cmプリント実行
//  ch1,ch2データ読み出しとプリント        
        cli();                         // 割り込み禁止にして
        for(i = 0; i < 2; i++){        // 2ch分のA/Dデータを取得
          d_max[i]  = ad_max[i];       // maxとmin
          d_min[i]  = ad_min[i];
          ad_max[i] = ad_min[i] = ad_data[i];    // 新データに
        }
        sei();                         // 割り込み再開
        if(digitalRead(SW_CH1) == HIGH)          // ch1
            pgrph(xpos(d_min[0]), xpos(d_max[0]));
        if(digitalRead(SW_CH2) == HIGH)          // ch2
            pgrph(xpos(d_min[1]), xpos(d_max[1]));
        prnpbff();                   // バッファをプリント
//  紙送りピッチ
        feed_8++;                    // 紙送りピッチ
        if(feed_8 >= 8){             // 1mm
            feed_8 = 0;
            feed_mm++;
            if(feed_mm >= 1000){
                feed_mm = 0;         // 0～999mm
            }
        }
        TP4_L;
    }
}

/*=====  end of prnsld01a    =====*/

